@@ -57,7 +57,10 @@ class GlancesBottle(object):
         self._route()
 
         # Path where the statics files are stored
-        self.STATIC_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'static')
+        self.STATIC_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'static/public')
+
+    def app(self):
+        return self._app()
 
     def check_auth(self, username, password):
         """Check if a username/password combination is valid."""
@@ -73,27 +76,25 @@ class GlancesBottle(object):
         self._app.route('/', method="GET", callback=self._index)
         self._app.route('/<refresh_time:int>', method=["GET"], callback=self._index)
 
-        self._app.route('/<filename:re:.*\.css>', method="GET", callback=self._css)
-        self._app.route('/<filename:re:.*\.js>', method="GET", callback=self._js)
-        self._app.route('/<filename:re:.*\.js.map>', method="GET", callback=self._js_map)
-        self._app.route('/<filename:re:.*\.html>', method="GET", callback=self._html)
-
-        self._app.route('/<filename:re:.*\.png>', method="GET", callback=self._images)
-        self._app.route('/favicon.ico', method="GET", callback=self._favicon)
-
         # REST API
         self._app.route('/api/2/args', method="GET", callback=self._api_args)
-        self._app.route('/api/2/args/:item', method="GET", callback=self._api_args_item)
+        self._app.route('/api/2/args/<item>', method="GET", callback=self._api_args_item)
         self._app.route('/api/2/help', method="GET", callback=self._api_help)
         self._app.route('/api/2/pluginslist', method="GET", callback=self._api_plugins)
         self._app.route('/api/2/all', method="GET", callback=self._api_all)
         self._app.route('/api/2/all/limits', method="GET", callback=self._api_all_limits)
         self._app.route('/api/2/all/views', method="GET", callback=self._api_all_views)
-        self._app.route('/api/2/:plugin', method="GET", callback=self._api)
-        self._app.route('/api/2/:plugin/limits', method="GET", callback=self._api_limits)
-        self._app.route('/api/2/:plugin/views', method="GET", callback=self._api_views)
-        self._app.route('/api/2/:plugin/:item', method="GET", callback=self._api_item)
-        self._app.route('/api/2/:plugin/:item/:value', method="GET", callback=self._api_value)
+        self._app.route('/api/2/<plugin>', method="GET", callback=self._api)
+        self._app.route('/api/2/<plugin>/history', method="GET", callback=self._api_history)
+        self._app.route('/api/2/<plugin>/history/<nb:int>', method="GET", callback=self._api_history)
+        self._app.route('/api/2/<plugin>/limits', method="GET", callback=self._api_limits)
+        self._app.route('/api/2/<plugin>/views', method="GET", callback=self._api_views)
+        self._app.route('/api/2/<plugin>/<item>', method="GET", callback=self._api_item)
+        self._app.route('/api/2/<plugin>/<item>/history', method="GET", callback=self._api_item_history)
+        self._app.route('/api/2/<plugin>/<item>/history/<nb:int>', method="GET", callback=self._api_item_history)
+        self._app.route('/api/2/<plugin>/<item>/<value>', method="GET", callback=self._api_value)
+
+        self._app.route('/<filepath:path>', method="GET", callback=self._resource)
 
     def start(self, stats):
         """Start the bottle."""
@@ -104,7 +105,7 @@ class GlancesBottle(object):
         self.plugins_list = self.stats.getAllPlugins()
 
         # Bind the Bottle TCP address/port
-        bindmsg = 'Glances web server started on http://{0}:{1}/'.format(self.args.bind_address, self.args.port)
+        bindmsg = 'Glances web server started on http://{}:{}/'.format(self.args.bind_address, self.args.port)
         logger.info(bindmsg)
         print(bindmsg)
         self._app.run(host=self.args.bind_address, port=self.args.port, quiet=not self.args.debug)
@@ -119,37 +120,12 @@ class GlancesBottle(object):
         self.stats.update()
 
         # Display
-        return static_file("index.html", root=os.path.join(self.STATIC_PATH, 'html'))
+        return static_file("index.html", root=self.STATIC_PATH)
 
-    def _html(self, filename):
-        """Bottle callback for *.html files."""
+    def _resource(self, filepath):
+        """Bottle callback for resources files."""
         # Return the static file
-        return static_file(filename, root=os.path.join(self.STATIC_PATH, 'html'))
-
-    def _css(self, filename):
-        """Bottle callback for *.css files."""
-        # Return the static file
-        return static_file(filename, root=os.path.join(self.STATIC_PATH, 'css'))
-
-    def _js(self, filename):
-        """Bottle callback for *.js files."""
-        # Return the static file
-        return static_file(filename, root=os.path.join(self.STATIC_PATH, 'js'))
-
-    def _js_map(self, filename):
-        """Bottle callback for *.js.map files."""
-        # Return the static file
-        return static_file(filename, root=os.path.join(self.STATIC_PATH, 'js'))
-
-    def _images(self, filename):
-        """Bottle callback for *.png files."""
-        # Return the static file
-        return static_file(filename, root=os.path.join(self.STATIC_PATH, 'images'))
-
-    def _favicon(self):
-        """Bottle callback for favicon."""
-        # Return the static file
-        return static_file('favicon.ico', root=self.STATIC_PATH)
+        return static_file(filepath, root=self.STATIC_PATH)
 
     def _api_help(self):
         """Glances API RESTFul implementation.
@@ -287,6 +263,30 @@ class GlancesBottle(object):
             abort(404, "Cannot get plugin %s (%s)" % (plugin, str(e)))
         return statval
 
+    def _api_history(self, plugin, nb=0):
+        """Glances API RESTFul implementation.
+
+        Return the JSON representation of a given plugin history
+        Limit to the last nb items (all if nb=0)
+        HTTP/200 if OK
+        HTTP/400 if plugin is not found
+        HTTP/404 if others error
+        """
+        response.content_type = 'application/json'
+
+        if plugin not in self.plugins_list:
+            abort(400, "Unknown plugin %s (available plugins: %s)" % (plugin, self.plugins_list))
+
+        # Update the stat
+        self.stats.update()
+
+        try:
+            # Get the JSON value of the stat ID
+            statval = self.stats.get_plugin(plugin).get_stats_history(nb=int(nb))
+        except Exception as e:
+            abort(404, "Cannot get plugin history %s (%s)" % (plugin, str(e)))
+        return statval
+
     def _api_limits(self, plugin):
         """Glances API RESTFul implementation.
 
@@ -333,8 +333,8 @@ class GlancesBottle(object):
             abort(404, "Cannot get views for plugin %s (%s)" % (plugin, str(e)))
         return ret
 
-    def _api_itemvalue(self, plugin, item, value=None):
-        """ Father method for _api_item and _api_value"""
+    def _api_itemvalue(self, plugin, item, value=None, history=False, nb=0):
+        """Father method for _api_item and _api_value"""
         response.content_type = 'application/json'
 
         if plugin not in self.plugins_list:
@@ -344,22 +344,29 @@ class GlancesBottle(object):
         self.stats.update()
 
         if value is None:
-            ret = self.stats.get_plugin(plugin).get_stats_item(item)
+            if history:
+                ret = self.stats.get_plugin(plugin).get_stats_history(item, nb=int(nb))
+            else:
+                ret = self.stats.get_plugin(plugin).get_stats_item(item)
 
             if ret is None:
-                abort(404, "Cannot get item %s in plugin %s" % (item, plugin))
+                abort(404, "Cannot get item %s%s in plugin %s" % (item, 'history ' if history else '', plugin))
         else:
-            ret = self.stats.get_plugin(plugin).get_stats_value(item, value)
+            if history:
+                # Not available
+                ret = None
+            else:
+                ret = self.stats.get_plugin(plugin).get_stats_value(item, value)
 
             if ret is None:
-                abort(404, "Cannot get item(%s)=value(%s) in plugin %s" % (item, value, plugin))
+                abort(404, "Cannot get item %s(%s=%s) in plugin %s" % ('history ' if history else '', item, value, plugin))
 
         return ret
 
     def _api_item(self, plugin, item):
         """Glances API RESTFul implementation.
 
-        Return the JSON represenation of the couple plugin/item
+        Return the JSON representation of the couple plugin/item
         HTTP/200 if OK
         HTTP/400 if plugin is not found
         HTTP/404 if others error
@@ -367,6 +374,17 @@ class GlancesBottle(object):
         """
         return self._api_itemvalue(plugin, item)
 
+    def _api_item_history(self, plugin, item, nb=0):
+        """Glances API RESTFul implementation.
+
+        Return the JSON representation of the couple plugin/history of item
+        HTTP/200 if OK
+        HTTP/400 if plugin is not found
+        HTTP/404 if others error
+
+        """
+        return self._api_itemvalue(plugin, item, history=True, nb=int(nb))
+
     def _api_value(self, plugin, item, value):
         """Glances API RESTFul implementation.
 
