@@ -30,8 +30,14 @@ import psutil
 # Define the history items list
 # All items in this list will be historised if the --enable-history tag is set
 # 'color' define the graph color in #RGB format
-items_history_list = [{'name': 'read_bytes', 'color': '#00FF00', 'y_unit': 'B/s'},
-                      {'name': 'write_bytes', 'color': '#FF0000', 'y_unit': 'B/s'}]
+items_history_list = [{'name': 'read_bytes',
+                       'description': 'Bytes read per second',
+                       'color': '#00FF00',
+                       'y_unit': 'B/s'},
+                      {'name': 'write_bytes',
+                       'description': 'Bytes write per second',
+                       'color': '#FF0000',
+                       'y_unit': 'B/s'}]
 
 
 class Plugin(GlancesPlugin):
@@ -81,7 +87,7 @@ class Plugin(GlancesPlugin):
 
             # Previous disk IO stats are stored in the diskio_old variable
             if not hasattr(self, 'diskio_old'):
-                # First call, we init the network_old var
+                # First call, we init the diskio_old var
                 try:
                     self.diskio_old = diskiocounters
                 except (IOError, UnboundLocalError):
@@ -119,6 +125,9 @@ class Plugin(GlancesPlugin):
                             'write_count': write_count,
                             'read_bytes': read_bytes,
                             'write_bytes': write_bytes}
+                        # Add alias if exist (define in the configuration file)
+                        if self.has_alias(disk) is not None:
+                            diskstat['alias'] = self.has_alias(disk)
                     except KeyError:
                         continue
                     else:
@@ -165,17 +174,17 @@ class Plugin(GlancesPlugin):
 
         # Build the string message
         # Header
-        msg = '{0:9}'.format('DISK I/O')
+        msg = '{:9}'.format('DISK I/O')
         ret.append(self.curse_add_line(msg, "TITLE"))
         if args.diskio_iops:
-            msg = '{0:>7}'.format('IOR/s')
+            msg = '{:>7}'.format('IOR/s')
             ret.append(self.curse_add_line(msg))
-            msg = '{0:>7}'.format('IOW/s')
+            msg = '{:>7}'.format('IOW/s')
             ret.append(self.curse_add_line(msg))
         else:
-            msg = '{0:>7}'.format('R/s')
+            msg = '{:>7}'.format('R/s')
             ret.append(self.curse_add_line(msg))
-            msg = '{0:>7}'.format('W/s')
+            msg = '{:>7}'.format('W/s')
             ret.append(self.curse_add_line(msg))
         # Disk list (sorted by name)
         for i in sorted(self.stats, key=operator.itemgetter(self.get_key())):
@@ -189,7 +198,7 @@ class Plugin(GlancesPlugin):
             if len(disk_name) > 9:
                 # Cut disk name if it is too long
                 disk_name = '_' + disk_name[-8:]
-            msg = '{0:9}'.format(disk_name)
+            msg = '{:9}'.format(disk_name)
             ret.append(self.curse_add_line(msg))
             if args.diskio_iops:
                 # count
@@ -197,12 +206,12 @@ class Plugin(GlancesPlugin):
                     int(i['read_count'] // i['time_since_update']))
                 rxps = self.auto_unit(
                     int(i['write_count'] // i['time_since_update']))
-                msg = '{0:>7}'.format(txps)
+                msg = '{:>7}'.format(txps)
                 ret.append(self.curse_add_line(msg,
                                                self.get_views(item=i[self.get_key()],
                                                               key='read_count',
                                                               option='decoration')))
-                msg = '{0:>7}'.format(rxps)
+                msg = '{:>7}'.format(rxps)
                 ret.append(self.curse_add_line(msg,
                                                self.get_views(item=i[self.get_key()],
                                                               key='write_count',
@@ -213,12 +222,12 @@ class Plugin(GlancesPlugin):
                     int(i['read_bytes'] // i['time_since_update']))
                 rxps = self.auto_unit(
                     int(i['write_bytes'] // i['time_since_update']))
-                msg = '{0:>7}'.format(txps)
+                msg = '{:>7}'.format(txps)
                 ret.append(self.curse_add_line(msg,
                                                self.get_views(item=i[self.get_key()],
                                                               key='read_bytes',
                                                               option='decoration')))
-                msg = '{0:>7}'.format(rxps)
+                msg = '{:>7}'.format(rxps)
                 ret.append(self.curse_add_line(msg,
                                                self.get_views(item=i[self.get_key()],
                                                               key='write_bytes',
