@@ -19,14 +19,13 @@
 
 """Process list plugin."""
 
-import operator
 import os
 from datetime import timedelta
 
 from glances.compat import iteritems
-from glances.globals import WINDOWS
+from glances.globals import LINUX, WINDOWS
 from glances.logger import logger
-from glances.processes import glances_processes
+from glances.processes import glances_processes, sort_stats
 from glances.plugins.glances_core import Plugin as CorePlugin
 from glances.plugins.glances_plugin import GlancesPlugin
 
@@ -42,6 +41,21 @@ def convert_timedelta(delta):
     return hours, minutes, seconds, microseconds
 
 
+def split_cmdline(cmdline):
+    """Return path, cmd and arguments for a process cmdline."""
+    path, cmd = os.path.split(cmdline[0])
+    arguments = ' '.join(cmdline[1:]).replace('\n', ' ')
+    # XXX: workaround for psutil issue #742
+    if LINUX and any(x in cmdline[0] for x in ('chrome', 'chromium')):
+        try:
+            exe, arguments = cmdline[0].split(' ', 1)
+            path, cmd = os.path.split(exe)
+        except ValueError:
+            arguments = None
+
+    return path, cmd, arguments
+
+
 class Plugin(GlancesPlugin):
 
     """Glances' processes plugin.
@@ -130,10 +144,9 @@ class Plugin(GlancesPlugin):
         # find process command indices in messages
         pos = []
         for i, m in enumerate(child_data):
-            if m["msg"] == "\n" and m is not child_data[-1]:
-                # new line pos + 12
-                # TODO find a way to get rid of hardcoded 12 value
-                pos.append(i + 12)
+            if m.get("_tree_decoration", False):
+                del m["_tree_decoration"]
+                pos.append(i)
 
         # add new curses items for tree decoration
         new_child_data = []
@@ -142,35 +155,38 @@ class Plugin(GlancesPlugin):
             if i in pos:
                 new_pos.append(len(new_child_data))
                 new_child_data.append(self.curse_add_line(""))
+                new_child_data[-1]["_tree_decoration"] = True
             new_child_data.append(m)
         child_data = new_child_data
         pos = new_pos
 
-        # draw node prefix
-        if is_last_child:
-            prefix = "└─"
-        else:
-            prefix = "├─"
-        child_data[pos[0]]["msg"] = prefix
+        if pos:
+            # draw node prefix
+            if is_last_child:
+                prefix = "└─"
+            else:
+                prefix = "├─"
+            child_data[pos[0]]["msg"] = prefix
 
-        # add indentation
-        for i in pos:
-            spacing = 2
-            if first_level:
-                spacing = 1
-            elif is_last_child and (i is not pos[0]):
-                # compensate indentation for missing '│' char
-                spacing = 3
-            child_data[i]["msg"] = "%s%s" % (" " * spacing, child_data[i]["msg"])
-
-        if not is_last_child:
-            # add '│' tree decoration
-            for i in pos[1:]:
-                old_str = child_data[i]["msg"]
+            # add indentation
+            for i in pos:
+                spacing = 2
                 if first_level:
-                    child_data[i]["msg"] = " │" + old_str[2:]
-                else:
-                    child_data[i]["msg"] = old_str[:2] + "│" + old_str[3:]
+                    spacing = 1
+                elif is_last_child and (i is not pos[0]):
+                    # compensate indentation for missing '│' char
+                    spacing = 3
+                child_data[i]["msg"] = "%s%s" % (" " * spacing, child_data[i]["msg"])
+
+            if not is_last_child:
+                # add '│' tree decoration
+                for i in pos[1:]:
+                    old_str = child_data[i]["msg"]
+                    if first_level:
+                        child_data[i]["msg"] = " │" + old_str[2:]
+                    else:
+                        child_data[i]["msg"] = old_str[:2] + "│" + old_str[3:]
+
         return child_data
 
     def get_process_curses_data(self, p, first, args):
@@ -179,69 +195,70 @@ class Plugin(GlancesPlugin):
         # CPU
         if 'cpu_percent' in p and p['cpu_percent'] is not None and p['cpu_percent'] != '':
             if args.disable_irix and self.nb_log_core != 0:
-                msg = '{0:>6.1f}'.format(p['cpu_percent'] / float(self.nb_log_core))
+                msg = '{:>6.1f}'.format(p['cpu_percent'] / float(self.nb_log_core))
             else:
-                msg = '{0:>6.1f}'.format(p['cpu_percent'])
+                msg = '{:>6.1f}'.format(p['cpu_percent'])
             ret.append(self.curse_add_line(msg,
                                            self.get_alert(p['cpu_percent'], header="cpu")))
         else:
-            msg = '{0:>6}'.format('?')
+            msg = '{:>6}'.format('?')
             ret.append(self.curse_add_line(msg))
         # MEM
         if 'memory_percent' in p and p['memory_percent'] is not None and p['memory_percent'] != '':
-            msg = '{0:>6.1f}'.format(p['memory_percent'])
+            msg = '{:>6.1f}'.format(p['memory_percent'])
             ret.append(self.curse_add_line(msg,
                                            self.get_alert(p['memory_percent'], header="mem")))
         else:
-            msg = '{0:>6}'.format('?')
+            msg = '{:>6}'.format('?')
             ret.append(self.curse_add_line(msg))
         # VMS/RSS
         if 'memory_info' in p and p['memory_info'] is not None and p['memory_info'] != '':
             # VMS
-            msg = '{0:>6}'.format(self.auto_unit(p['memory_info'][1], low_precision=False))
+            msg = '{:>6}'.format(self.auto_unit(p['memory_info'][1], low_precision=False))
             ret.append(self.curse_add_line(msg, optional=True))
             # RSS
-            msg = '{0:>6}'.format(self.auto_unit(p['memory_info'][0], low_precision=False))
+            msg = '{:>6}'.format(self.auto_unit(p['memory_info'][0], low_precision=False))
             ret.append(self.curse_add_line(msg, optional=True))
         else:
-            msg = '{0:>6}'.format('?')
+            msg = '{:>6}'.format('?')
             ret.append(self.curse_add_line(msg))
             ret.append(self.curse_add_line(msg))
         # PID
-        msg = '{0:>6}'.format(p['pid'])
+        msg = '{:>6}'.format(p['pid'])
         ret.append(self.curse_add_line(msg))
         # USER
         if 'username' in p:
             # docker internal users are displayed as ints only, therefore str()
-            msg = ' {0:9}'.format(str(p['username'])[:9])
+            # Correct issue #886 on Windows OS
+            msg = ' {:9}'.format(str(p['username'])[:9])
             ret.append(self.curse_add_line(msg))
         else:
-            msg = ' {0:9}'.format('?')
+            msg = ' {:9}'.format('?')
             ret.append(self.curse_add_line(msg))
         # NICE
         if 'nice' in p:
             nice = p['nice']
             if nice is None:
                 nice = '?'
-            msg = '{0:>5}'.format(nice)
+            msg = '{:>5}'.format(nice)
             if isinstance(nice, int) and ((WINDOWS and nice != 32) or
                                           (not WINDOWS and nice != 0)):
                 ret.append(self.curse_add_line(msg, decoration='NICE'))
             else:
                 ret.append(self.curse_add_line(msg))
         else:
-            msg = '{0:>5}'.format('?')
+            msg = '{:>5}'.format('?')
             ret.append(self.curse_add_line(msg))
         # STATUS
         if 'status' in p:
             status = p['status']
-            msg = '{0:>2}'.format(status)
+            msg = '{:>2}'.format(status)
             if status == 'R':
                 ret.append(self.curse_add_line(msg, decoration='STATUS'))
             else:
                 ret.append(self.curse_add_line(msg))
         else:
-            msg = '{0:>2}'.format('?')
+            msg = '{:>2}'.format('?')
             ret.append(self.curse_add_line(msg))
         # TIME+
         if self.tag_proc_time:
@@ -252,37 +269,37 @@ class Plugin(GlancesPlugin):
                 # See https://github.com/nicolargo/glances/issues/87
                 # Also catch TypeError on Mac OS X
                 # See: https://github.com/nicolargo/glances/issues/622
-                logger.debug("Cannot get TIME+ ({0})".format(e))
+                logger.debug("Cannot get TIME+ ({})".format(e))
                 self.tag_proc_time = False
             else:
                 hours, minutes, seconds, microseconds = convert_timedelta(delta)
                 if hours:
-                    msg = '{0:>4}h'.format(hours)
+                    msg = '{:>4}h'.format(hours)
                     ret.append(self.curse_add_line(msg, decoration='CPU_TIME', optional=True))
-                    msg = '{0}:{1}'.format(str(minutes).zfill(2), seconds)
+                    msg = '{}:{}'.format(str(minutes).zfill(2), seconds)
                 else:
-                    msg = '{0:>4}:{1}.{2}'.format(minutes, seconds, microseconds)
+                    msg = '{:>4}:{}.{}'.format(minutes, seconds, microseconds)
         else:
-            msg = '{0:>10}'.format('?')
+            msg = '{:>10}'.format('?')
         ret.append(self.curse_add_line(msg, optional=True))
         # IO read/write
         if 'io_counters' in p:
             # IO read
             io_rs = int((p['io_counters'][0] - p['io_counters'][2]) / p['time_since_update'])
             if io_rs == 0:
-                msg = '{0:>6}'.format("0")
+                msg = '{:>6}'.format("0")
             else:
-                msg = '{0:>6}'.format(self.auto_unit(io_rs, low_precision=True))
+                msg = '{:>6}'.format(self.auto_unit(io_rs, low_precision=True))
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
             # IO write
             io_ws = int((p['io_counters'][1] - p['io_counters'][3]) / p['time_since_update'])
             if io_ws == 0:
-                msg = '{0:>6}'.format("0")
+                msg = '{:>6}'.format("0")
             else:
-                msg = '{0:>6}'.format(self.auto_unit(io_ws, low_precision=True))
+                msg = '{:>6}'.format(self.auto_unit(io_ws, low_precision=True))
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
         else:
-            msg = '{0:>6}'.format("?")
+            msg = '{:>6}'.format("?")
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
 
@@ -293,19 +310,25 @@ class Plugin(GlancesPlugin):
         try:
             # XXX: remove `cmdline != ['']` when we'll drop support for psutil<4.0.0
             if cmdline and cmdline != ['']:
-                path, cmd = os.path.split(cmdline[0])
+                path, cmd, arguments = split_cmdline(cmdline)
                 if os.path.isdir(path) and not args.process_short_name:
-                    msg = ' {0}'.format(path) + os.sep
+                    msg = ' {}'.format(path) + os.sep
                     ret.append(self.curse_add_line(msg, splittable=True))
+                    if glances_processes.is_tree_enabled():
+                        # mark position to add tree decoration
+                        ret[-1]["_tree_decoration"] = True
                     ret.append(self.curse_add_line(cmd, decoration='PROCESS', splittable=True))
                 else:
-                    msg = ' {0}'.format(cmd)
+                    msg = ' {}'.format(cmd)
                     ret.append(self.curse_add_line(msg, decoration='PROCESS', splittable=True))
-                arguments = ' '.join(cmdline[1:]).replace('\n', ' ')
-                msg = ' {0}'.format(arguments)
-                ret.append(self.curse_add_line(msg, splittable=True))
+                    if glances_processes.is_tree_enabled():
+                        # mark position to add tree decoration
+                        ret[-1]["_tree_decoration"] = True
+                if arguments:
+                    msg = ' {}'.format(arguments)
+                    ret.append(self.curse_add_line(msg, splittable=True))
             else:
-                msg = ' {0}'.format(p['name'])
+                msg = ' {}'.format(p['name'])
                 ret.append(self.curse_add_line(msg, splittable=True))
         except UnicodeEncodeError:
             ret.append(self.curse_add_line('', splittable=True))
@@ -322,10 +345,10 @@ class Plugin(GlancesPlugin):
                 msg = xpad + 'CPU affinity: ' + str(len(p['cpu_affinity'])) + ' cores'
                 ret.append(self.curse_add_line(msg, splittable=True))
             # Second line is memory info
-            if 'memory_info_ex' in p and p['memory_info_ex'] is not None:
+            if 'memory_info' in p and p['memory_info'] is not None:
                 ret.append(self.curse_new_line())
                 msg = xpad + 'Memory info: '
-                for k, v in iteritems(p['memory_info_ex']._asdict()):
+                for k, v in iteritems(p['memory_info']._asdict()):
                     # Ignore rss and vms (already displayed)
                     if k not in ['rss', 'vms'] and v is not None:
                         msg += k + ' ' + self.auto_unit(v, low_precision=False) + ' '
@@ -402,12 +425,12 @@ class Plugin(GlancesPlugin):
         # Process list
         if glances_processes.is_tree_enabled():
             ret.extend(self.get_process_tree_curses_data(
-                self.sort_stats(process_sort_key), args, first_level=True,
+                self.__sort_stats(process_sort_key), args, first_level=True,
                 max_node_count=glances_processes.max_processes))
         else:
             # Loop over processes (sorted by the sort key previously compute)
             first = True
-            for p in self.sort_stats(process_sort_key):
+            for p in self.__sort_stats(process_sort_key):
                 ret.extend(self.get_process_curses_data(p, first, args))
                 # End of extended stats
                 first = False
@@ -429,33 +452,33 @@ class Plugin(GlancesPlugin):
         sort_style = 'SORT'
 
         if args.disable_irix and 0 < self.nb_log_core < 10:
-            msg = '{0:>6}'.format('CPU%/' + str(self.nb_log_core))
+            msg = '{:>6}'.format('CPU%/' + str(self.nb_log_core))
         elif args.disable_irix and self.nb_log_core != 0:
-            msg = '{0:>6}'.format('CPU%/C')
+            msg = '{:>6}'.format('CPU%/C')
         else:
-            msg = '{0:>6}'.format('CPU%')
+            msg = '{:>6}'.format('CPU%')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'cpu_percent' else 'DEFAULT'))
-        msg = '{0:>6}'.format('MEM%')
+        msg = '{:>6}'.format('MEM%')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'memory_percent' else 'DEFAULT'))
-        msg = '{0:>6}'.format('VIRT')
+        msg = '{:>6}'.format('VIRT')
         ret.append(self.curse_add_line(msg, optional=True))
-        msg = '{0:>6}'.format('RES')
+        msg = '{:>6}'.format('RES')
         ret.append(self.curse_add_line(msg, optional=True))
-        msg = '{0:>6}'.format('PID')
+        msg = '{:>6}'.format('PID')
         ret.append(self.curse_add_line(msg))
-        msg = ' {0:10}'.format('USER')
+        msg = ' {:10}'.format('USER')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'username' else 'DEFAULT'))
-        msg = '{0:>4}'.format('NI')
+        msg = '{:>4}'.format('NI')
         ret.append(self.curse_add_line(msg))
-        msg = '{0:>2}'.format('S')
+        msg = '{:>2}'.format('S')
         ret.append(self.curse_add_line(msg))
-        msg = '{0:>10}'.format('TIME+')
+        msg = '{:>10}'.format('TIME+')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'cpu_times' else 'DEFAULT', optional=True))
-        msg = '{0:>6}'.format('R/s')
+        msg = '{:>6}'.format('R/s')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True))
-        msg = '{0:>6}'.format('W/s')
+        msg = '{:>6}'.format('W/s')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True))
-        msg = ' {0:8}'.format('Command')
+        msg = ' {:8}'.format('Command')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'name' else 'DEFAULT'))
 
     def __msg_curse_sum(self, ret, sep_char='_', mmm=None, args=None):
@@ -471,73 +494,73 @@ class Plugin(GlancesPlugin):
             ret.append(self.curse_add_line(sep_char * 69))
             ret.append(self.curse_new_line())
         # CPU percent sum
-        msg = '{0:>6.1f}'.format(self.__sum_stats('cpu_percent', mmm=mmm))
+        msg = '{:>6.1f}'.format(self.__sum_stats('cpu_percent', mmm=mmm))
         ret.append(self.curse_add_line(msg,
                                        decoration=self.__mmm_deco(mmm)))
         # MEM percent sum
-        msg = '{0:>6.1f}'.format(self.__sum_stats('memory_percent', mmm=mmm))
+        msg = '{:>6.1f}'.format(self.__sum_stats('memory_percent', mmm=mmm))
         ret.append(self.curse_add_line(msg,
                                        decoration=self.__mmm_deco(mmm)))
         # VIRT and RES memory sum
         if 'memory_info' in self.stats[0] and self.stats[0]['memory_info'] is not None and self.stats[0]['memory_info'] != '':
             # VMS
-            msg = '{0:>6}'.format(self.auto_unit(self.__sum_stats('memory_info', indice=1, mmm=mmm), low_precision=False))
+            msg = '{:>6}'.format(self.auto_unit(self.__sum_stats('memory_info', indice=1, mmm=mmm), low_precision=False))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True))
             # RSS
-            msg = '{0:>6}'.format(self.auto_unit(self.__sum_stats('memory_info', indice=0, mmm=mmm), low_precision=False))
+            msg = '{:>6}'.format(self.auto_unit(self.__sum_stats('memory_info', indice=0, mmm=mmm), low_precision=False))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True))
         else:
-            msg = '{0:>6}'.format('')
+            msg = '{:>6}'.format('')
             ret.append(self.curse_add_line(msg))
             ret.append(self.curse_add_line(msg))
         # PID
-        msg = '{0:>6}'.format('')
+        msg = '{:>6}'.format('')
         ret.append(self.curse_add_line(msg))
         # USER
-        msg = ' {0:9}'.format('')
+        msg = ' {:9}'.format('')
         ret.append(self.curse_add_line(msg))
         # NICE
-        msg = '{0:>5}'.format('')
+        msg = '{:>5}'.format('')
         ret.append(self.curse_add_line(msg))
         # STATUS
-        msg = '{0:>2}'.format('')
+        msg = '{:>2}'.format('')
         ret.append(self.curse_add_line(msg))
         # TIME+
-        msg = '{0:>10}'.format('')
+        msg = '{:>10}'.format('')
         ret.append(self.curse_add_line(msg, optional=True))
         # IO read/write
         if 'io_counters' in self.stats[0] and mmm is None:
             # IO read
             io_rs = int((self.__sum_stats('io_counters', 0) - self.__sum_stats('io_counters', indice=2, mmm=mmm)) / self.stats[0]['time_since_update'])
             if io_rs == 0:
-                msg = '{0:>6}'.format('0')
+                msg = '{:>6}'.format('0')
             else:
-                msg = '{0:>6}'.format(self.auto_unit(io_rs, low_precision=True))
+                msg = '{:>6}'.format(self.auto_unit(io_rs, low_precision=True))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True, additional=True))
             # IO write
             io_ws = int((self.__sum_stats('io_counters', 1) - self.__sum_stats('io_counters', indice=3, mmm=mmm)) / self.stats[0]['time_since_update'])
             if io_ws == 0:
-                msg = '{0:>6}'.format('0')
+                msg = '{:>6}'.format('0')
             else:
-                msg = '{0:>6}'.format(self.auto_unit(io_ws, low_precision=True))
+                msg = '{:>6}'.format(self.auto_unit(io_ws, low_precision=True))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True, additional=True))
         else:
-            msg = '{0:>6}'.format('')
+            msg = '{:>6}'.format('')
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
         if mmm is None:
-            msg = ' < {0}'.format('current')
+            msg = ' < {}'.format('current')
             ret.append(self.curse_add_line(msg, optional=True))
         else:
-            msg = ' < {0}'.format(mmm)
+            msg = ' < {}'.format(mmm)
             ret.append(self.curse_add_line(msg, optional=True))
             msg = ' (\'M\' to reset)'
             ret.append(self.curse_add_line(msg, optional=True))
@@ -603,36 +626,8 @@ class Plugin(GlancesPlugin):
             ret += str(indice)
         return ret
 
-    def sort_stats(self, sortedby=None):
-        """Return the stats sorted by sortedby variable."""
-        if sortedby is None:
-            # No need to sort...
-            return self.stats
-
-        tree = glances_processes.is_tree_enabled()
-
-        if sortedby == 'io_counters' and not tree:
-            # Specific case for io_counters
-            # Sum of io_r + io_w
-            try:
-                # Sort process by IO rate (sum IO read + IO write)
-                self.stats.sort(key=lambda process: process[sortedby][0] -
-                                process[sortedby][2] + process[sortedby][1] -
-                                process[sortedby][3],
-                                reverse=glances_processes.sort_reverse)
-            except Exception:
-                self.stats.sort(key=operator.itemgetter('cpu_percent'),
-                                reverse=glances_processes.sort_reverse)
-        else:
-            # Others sorts
-            if tree:
-                self.stats.set_sorting(sortedby, glances_processes.sort_reverse)
-            else:
-                try:
-                    self.stats.sort(key=operator.itemgetter(sortedby),
-                                    reverse=glances_processes.sort_reverse)
-                except (KeyError, TypeError):
-                    self.stats.sort(key=operator.itemgetter('name'),
-                                    reverse=False)
-
-        return self.stats
+    def __sort_stats(self, sortedby=None):
+        """Return the stats (dict) sorted by (sortedby)"""
+        return sort_stats(self.stats, sortedby,
+                          tree=glances_processes.is_tree_enabled(),
+                          reverse=glances_processes.sort_reverse)
