@@ -26,6 +26,7 @@ from glances.logger import logger
 from glances.processes import glances_processes
 from glances.stats import GlancesStats
 from glances.outputs.glances_curses import GlancesCursesStandalone
+from glances.outdated import Outdated
 
 
 class GlancesStandalone(object):
@@ -75,7 +76,10 @@ class GlancesStandalone(object):
             glances_processes.max_processes = 50
 
             # Init screen
-            self.screen = GlancesCursesStandalone(args=args)
+            self.screen = GlancesCursesStandalone(config=config, args=args)
+
+        # Check the latest Glances version
+        self.outdated = Outdated(config=config, args=args)
 
     @property
     def quiet(self):
@@ -115,3 +119,9 @@ class GlancesStandalone(object):
 
         # Exit from export modules
         self.stats.end()
+
+        # Check Glances version versus Pypi one
+        if self.outdated.is_outdated():
+            print("You are using Glances version {}, however version {} is available.".format(
+                self.outdated.installed_version(), self.outdated.latest_version()))
+            print("You should consider upgrading using: pip install --upgrade glances")
